package io.tswf.gpt.google.gemini

import feign.okhttp.OkHttpClient
import groovy.transform.CompileStatic
import groovy.transform.Memoized
import io.tswf.gpt.google.gemini.api.ApiClient
import io.tswf.gpt.google.gemini.api.client.GeminiChatCompletionsApi
import io.tswf.gpt.google.gemini.api.client.GeminiTokenizationApi
import io.tswf.groovy.bettergroovy.transforms.nullsafe.NullSafe
import okhttp3.Credentials
import okhttp3.OkHttpClient.Builder as OkHttpClientBuilder

@CompileStatic
class GeminiApi {

    @NullSafe
    String googleApiToken

    @NullSafe
    String geminiBaseUrl = System.findGlobalProperty("gemini.api.base-url")
        .orElse(GeminiURLs.GEMINI_DEFAULT_BASE_PATH)

    Proxy geminiProxy = null

    String geminiProxyUser = null

    String geminiProxyPassword = null

    GeminiApi(@NullSafe String googleApiToken, @DelegatesTo(GeminiApi) Closure configurator = {  }) {
        this.googleApiToken = googleApiToken
        if (configurator != null) {
            configurator.delegate = this
            configurator.call(this)
        }
    }

    @Memoized
    GeminiTokenizationApi getTokenizationApi() {
            return apiClientsBuilder.buildClient(GeminiTokenizationApi)
    }

    @Memoized
    GeminiChatCompletionsApi getChatCompletionsApi() {
        return apiClientsBuilder.buildClient(GeminiChatCompletionsApi)
    }

    @Memoized
    ApiClient getApiClientsBuilder() {
        return new ApiClient()
                .tap {
                    feignBuilder.tap {

                        // Гугловая авторизация на API
                        requestInterceptor { request ->
                            // API ключ гугла
                            request.query("key", googleApiToken)
                        }

                        client(
                            new OkHttpClient(
                                new OkHttpClientBuilder()
                                    .tap {
                                        geminiProxy?.let {
                                            proxy(it)
                                            // Авторизация для прокси, если требуется
                                            if (geminiProxyUser.isNotNullOrBlank()) {
                                                if (geminiProxyPassword == null) {
                                                    throw new IllegalStateException("Password was not given for proxy user with username: '${geminiProxyUser}'")
                                                }

                                                if (geminiProxy == null) {
                                                    throw new IllegalStateException("Proxy password set when proxy is null!")
                                                }

                                                if (geminiProxy.type() != Proxy.Type.HTTP) {
                                                    throw new IllegalStateException("Proxy authorization is only supported for HTTP proxy type, not for '${geminiProxy.type()?.name()}'!")
                                                }

                                                proxyAuthenticator { route, response ->
                                                    response.request().newBuilder()
                                                        .header("Proxy-Authorization", Credentials.basic(geminiProxyUser, geminiProxyPassword))
                                                        .build()
                                                }
                                            }
                                        }
                                    }
                                .build()
                            )
                        )
                    }

                    basePath = geminiBaseUrl
                }
    }
}
